# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/XX_tmpl.ipynb.

# %% auto 0
__all__ = []

# %% ../nbs/XX_tmpl.ipynb 6
#| export


# %% ../nbs/XX_tmpl.ipynb 8
from typing import List, Union, TypeAlias, Callable, Optional, ParamSpec, Type, Callable, Tuple

# %% ../nbs/XX_tmpl.ipynb 10
#| export


# %% ../nbs/XX_tmpl.ipynb 12
#| export


# %% ../nbs/XX_tmpl.ipynb 14
#| export


# %% ../nbs/XX_tmpl.ipynb 16
#| export

